@@ -7,6 +7,7 @@ from typing import Union, Iterable, Optional, List, Tuple
 from collections.abc import Iterable as Iterabc
 from .metrics import beta_div
 from tqdm import tqdm
+from .constants import eps
 
 _size_1_t = Union[int, Tuple[int]]
 _size_2_t = Union[int, Tuple[int, int]]
@@ -16,7 +17,6 @@ __all__ = [
     'BaseComponent', 'NMF', 'NMFD', 'NMF2D', 'NMF3D'
 ]
 
-eps = 1e-8
 
 def _proj_func(s: Tensor,
                k1: float,
@@ -60,8 +60,11 @@ def _double_backward_update(V: Tensor,
         output_neg = V
         output_pos = WH
     elif beta == 1:
-        output_neg = V / WH
+        output_neg = V / WH.add(eps)
         output_pos = None
+    elif beta == 0:
+        output_neg = V / (WH * WH).add(eps)
+        output_pos = 1 / WH.add(eps)
     else:
         output_neg = WH.pow(beta - 2) * V
         output_pos = WH.pow(beta - 1)
@@ -77,7 +80,7 @@ def _double_backward_update(V: Tensor,
     if l1_reg > 0:
         pos.add_(l1_reg)
     if l2_reg > 0:
-        pos.add_(param.data, alpha=l2_reg)
+        pos = pos.add(param.data, alpha=l2_reg)
     multiplier = neg / pos
     if gamma != 1:
         multiplier.pow_(gamma)
@@ -109,13 +112,13 @@ def _renorm(W: Tensor,
             H: Tensor,
             unit_norm='W'):
     if unit_norm == 'W':
-        W_norm = BaseComponent.get_W_norm(W)
+        W_norm = _get_norm(W)
         slicer = (slice(None),) + (None,) * (W.dim() - 2)
         W /= W_norm[slicer]
         slicer = (slice(None),) + (None,) * (H.dim() - 2)
         H *= W_norm[slicer]
     elif unit_norm == 'H':
-        H_norm = BaseComponent.get_H_norm(H)
+        H_norm = _get_norm(H)
         slicer = (slice(None),) + (None,) * (H.dim() - 2)
         H /= H_norm[slicer]
         slicer = (slice(None),) + (None,) * (W.dim() - 2)
@@ -131,17 +134,17 @@ class BaseComponent(torch.nn.Module):
     Your models should also subclass this class.
 
     Args:
-        rank (int): Size of hidden dimension
-        W (tuple or Tensor): Size or initial weights of template tensor W
-        H (tuple or Tensor): Size or initial weights of activation tensor H
-        trainable_W (bool):  If ``True``, the template tensor W is learnable. Default: ``True``
-        trainable_H (bool):  If ``True``, the activation tensor H is learnable. Default: ``True``
+        rank (int): size of hidden dimension
+        W (tuple or Tensor): size or initial weights of template tensor W
+        H (tuple or Tensor): size or initial weights of activation tensor H
+        trainable_W (bool):  controls whether template tensor W is trainable when initial weights is given. Default: ``True``
+        trainable_H (bool):  controls whether activation tensor H is trainable when initial weights is given. Default: ``True``
 
     Attributes:
         W (Tensor or None): the template tensor of the module if corresponding argument is given.
-            The values are initialized non-negatively.
+            If size is given, values are initialized non-negatively.
         H (Tensor or None): the activation tensor of the module if corresponding argument is given.
-            The values are initialized non-negatively.
+            If size is given, values are initialized non-negatively.
 
        """
     __constants__ = ['rank']
@@ -164,34 +167,40 @@ class BaseComponent(torch.nn.Module):
                  trainable_H: bool = True):
         super().__init__()
 
+        infer_rank = None
         if isinstance(W, Tensor):
-            self.register_parameter('W', Parameter(torch.empty(*W.size()), requires_grad=trainable_W))
+            assert torch.all(W >= 0.), "Tensor W should be non-negative."
+            self.register_parameter('W', Parameter(
+                torch.empty(*W.size()), requires_grad=trainable_W))
             self.W.data.copy_(W)
-        elif isinstance(W, Iterabc) and trainable_W:
+            infer_rank = self.W.shape[1]
+        elif isinstance(W, Iterabc):
             self.register_parameter('W', Parameter(torch.randn(*W).abs()))
+            infer_rank = W[1]
         else:
             self.register_parameter('W', None)
 
         if isinstance(H, Tensor):
+            assert torch.all(H >= 0.), "Tensor H should be non-negative."
             H_shape = H.shape
-            self.register_parameter('H', Parameter(torch.empty(*H_shape), requires_grad=trainable_H))
+            self.register_parameter('H', Parameter(
+                torch.empty(*H_shape), requires_grad=trainable_H))
             self.H.data.copy_(H)
-        elif isinstance(H, Iterabc) and trainable_H:
+            infer_rank = self.H.shape[1]
+        elif isinstance(H, Iterabc):
             self.register_parameter('H', Parameter(torch.randn(*H).abs()))
+            infer_rank = H[1]
         else:
             self.register_parameter('H', None)
 
-        if isinstance(self.W, Tensor):
-            if isinstance(self.H, Tensor):
-                assert self.W.shape[1] == self.H.shape[1], "Latent size of W and H should be equal!"
-            rank = self.W.shape[1]
-            self.out_channels = self.W.shape[0]
-            if len(self.W.shape) > 2:
-                self.kernel_size = tuple(self.W.shape[2:])
-        elif isinstance(self.H, Tensor):
-            rank = self.H.shape[1]
+        if infer_rank is None:
+            assert rank, "A rank should be given when W and H are not available!"
         else:
-            assert rank, "A rank should be given when both W and H are not available!"
+            if getattr(self, "H") is not None:
+                assert self.H.shape[1] == infer_rank, "Latent size of H does not match with others!"
+            if getattr(self, "W") is not None:
+                assert self.W.shape[1] == infer_rank, "Latent size of W does not match with others!"
+            rank = infer_rank
 
         self.rank = rank
 
@@ -213,12 +222,12 @@ class BaseComponent(torch.nn.Module):
 
         Args:
             H(Tensor, optional): input activation tensor H. If no tensor was given will use :attr:`H` from this module
-                                instead.
+                                instead
             W(Tensor, optional): input template tensor W. If no tensor was given will use :attr:`W` from this module
-                                instead.
+                                instead
 
         Returns:
-            tensor
+            Tensor: tensor
         """
         if H is None:
             H = self.H
@@ -246,25 +255,26 @@ class BaseComponent(torch.nn.Module):
         r"""Learn a NMF model for the data V by minimizing beta divergence.
 
         To invoke this function, attributes :meth:`H <torchnmf.nmf.BaseComponent.H>` and
-        :meth:`W <torchnmf.nmf.BaseComponent.H>` should be presented in this module.
+        :meth:`W <torchnmf.nmf.BaseComponent.W>` should be presented in this module.
 
         Args:
-            V (Tensor): data tensor to be decomposed.
+            V (Tensor): data tensor to be decomposed
             beta (float): beta divergence to be minimized, measuring the distance between V and the NMF model.
-                        Default: ``1.``.
-            tol (float): tolerance of the stopping condition. Default: ``1e-4``.
-            max_iter (int): maximum number of iterations before timing out. Default: ``200``.
-            verbose (bool): whether to be verbose. Default: ``False``.
-            alpha (float): constant that multiplies the regularization terms. Set it to zero to have no regularization.
-                            Default: ``0``.
+                        Default: ``1.``
+            tol (float): tolerance of the stopping condition. Default: ``1e-4``
+            max_iter (int): maximum number of iterations before timing out. Default: ``200``
+            verbose (bool): whether to be verbose. Default: ``False``
+            alpha (float): constant that multiplies the regularization terms. Set it to zero to have no regularization
+                            Default: ``0``
             l1_ratio (float):  the regularization mixing parameter, with 0 <= l1_ratio <= 1.
                                 For l1_ratio = 0 the penalty is an elementwise L2 penalty (aka Frobenius Norm).
                                 For l1_ratio = 1 it is an elementwise L1 penalty.
-                                For 0 < l1_ratio < 1, the penalty is a combination of L1 and L2. Default: ``0``.
+                                For 0 < l1_ratio < 1, the penalty is a combination of L1 and L2. Default: ``0``
 
         Returns:
-            total number of iterations.
+            int: total number of iterations
         """
+        assert torch.all(V >= 0.), "Target should be non-negative."
 
         W = self.W
         H = self.H
@@ -281,7 +291,8 @@ class BaseComponent(torch.nn.Module):
 
         with torch.no_grad():
             WH = self.reconstruct(H, W)
-            loss_init = previous_loss = beta_div(WH, V, beta).mul(2).sqrt().item()
+            loss_init = previous_loss = beta_div(
+                WH, V, beta).mul(2).sqrt().item()
 
         with tqdm(total=max_iter, disable=not verbose) as pbar:
             for n_iter in range(max_iter):
@@ -305,7 +316,7 @@ class BaseComponent(torch.nn.Module):
                         break
                     previous_loss = loss
 
-        return n_iter
+        return n_iter + 1
 
     def sparse_fit(self,
                    V,
@@ -319,24 +330,24 @@ class BaseComponent(torch.nn.Module):
         `Non-negative Matrix Factorization with Sparseness Constraints`_.
 
         To invoke this function, attributes :meth:`H <torchnmf.nmf.BaseComponent.H>` and
-        :meth:`W <torchnmf.nmf.BaseComponent.H>` should be presented in this module.
+        :meth:`W <torchnmf.nmf.BaseComponent.W>` should be presented in this module.
 
         .. _`Non-negative Matrix Factorization with Sparseness Constraints`:
             https://www.jmlr.org/papers/volume5/hoyer04a/hoyer04a.pdf
 
         Args:
-            V (Tensor): data tensor to be decomposed.
-            beta (float): beta divergence to be minimized, measuring the distance between V and the NMF model.
-                        Default: ``1.``.
-            max_iter (int): maximum number of iterations before timing out. Default: ``200``.
-            verbose (bool): whether to be verbose. Default: ``False``.
+            V (Tensor): data tensor to be decomposed
+            beta (float): beta divergence to be minimized, measuring the distance between V and the NMF model
+                        Default: ``1.``
+            max_iter (int): maximum number of iterations before timing out. Default: ``200``
+            verbose (bool): whether to be verbose. Default: ``False``
             sW (float or None): the target sparseness for template tensor :attr:`W` , with 0 < sW < 1. Set it to ``None``
                 will have no constraint. Default: ``None``
             sH (float or None): the target sparseness for activation tensor :attr:`H` , with 0 < sH < 1. Set it to ``None``
                 will have no constraint. Default: ``None``
 
         Returns:
-            total number of iterations.
+            int: total number of iterations
         """
         W = self.W
         H = self.H
@@ -377,15 +388,16 @@ class BaseComponent(torch.nn.Module):
                     else:
                         W.grad = None
                         WH = self.reconstruct(H.detach(), W)
-                        loss = beta_div(self.fix_neg(WH), V, beta)
+                        loss = beta_div(WH, V, beta)
                         loss.backward()
                         with torch.no_grad():
                             for i in range(10):
                                 Wnew = W - stepsize_W * W.grad
                                 norms = _get_norm(Wnew)
                                 for j in range(Wnew.shape[1]):
-                                    Wnew[:, j] = _proj_func(Wnew[:, j], L1a * norms[j], norms[j] ** 2)
-                                new_loss = beta_div(self.fix_neg(self.reconstruct(self.H, Wnew)),
+                                    Wnew[:, j] = _proj_func(
+                                        Wnew[:, j], L1a * norms[j], norms[j] ** 2)
+                                new_loss = beta_div(self.reconstruct(self.H, Wnew),
                                                     V, beta)
                                 if new_loss <= loss:
                                     break
@@ -400,10 +412,11 @@ class BaseComponent(torch.nn.Module):
                         WH = self.reconstruct(H, W.detach())
                         _double_backward_update(V, WH, H, beta, gamma, 0, 0,
                                                 _get_H_kl_positive(W.detach()) if beta == 1 else None)
+                        _renorm(W, H, 'H')
                     else:
                         H.grad = None
                         WH = self.reconstruct(H, W.detach())
-                        loss = beta_div(self.fix_neg(WH), V, beta)
+                        loss = beta_div(WH, V, beta)
                         loss.backward()
 
                         with torch.no_grad():
@@ -411,8 +424,9 @@ class BaseComponent(torch.nn.Module):
                                 Hnew = H - stepsize_H * H.grad
                                 norms = _get_norm(Hnew)
                                 for j in range(H.shape[1]):
-                                    Hnew[:, j] = _proj_func(Hnew[:, j], L1s * norms[j], norms[j] ** 2)
-                                new_loss = beta_div(self.fix_neg(self.reconstruct(Hnew, W)),
+                                    Hnew[:, j] = _proj_func(
+                                        Hnew[:, j], L1s * norms[j], norms[j] ** 2)
+                                new_loss = beta_div(self.reconstruct(Hnew, W),
                                                     V, beta)
                                 if new_loss <= loss:
                                     break
@@ -422,33 +436,31 @@ class BaseComponent(torch.nn.Module):
                             stepsize_H *= 1.2
                             H.copy_(Hnew)
 
-                        _renorm(W, H, 'W')
-
                 if n_iter % 10 == 9:
                     with torch.no_grad():
                         WH = self.reconstruct(H, W)
                         loss = beta_div(WH, V, beta).mul(2).sqrt().item()
                     pbar.set_postfix(loss=loss)
                     pbar.update(10)
-        return n_iter
+        return n_iter + 1
 
 
 class NMF(BaseComponent):
     r"""Non-Negative Matrix Factorization (NMF).
 
     Find two non-negative matrices (W, H) whose product approximates the non-
-    negative matrix V: :math:`V \approx WH`
+    negative matrix V: :math:`V \approx HW^T`.
 
     This factorization can be used for example for dimensionality reduction, source separation or topic extraction.
 
     Note:
-        To match with PyTorch convention, this class actually use :math:`H^T` so the batch dimension is the first
-        dimension, and require input target matrix V is also transposed.
+        If `Vshape` argument is given, the model will try to infer the size of :meth:`W <torchnmf.nmf.BaseComponent.W>` and
+        :meth:`H <torchnmf.nmf.BaseComponent.H>`, and override arguments passed through to :meth:`BaseComponent <torchnmf.nmf.BaseComponent>`.
 
     Args:
-        Vshape (tuple, optional): Size of target matrix V
-        rank (int, optional): Size of hidden dimension
-        **kwargs: Arguments passed through to :meth:`BaseComponent <torchnmf.nmf.BaseComponent>`.
+        Vshape (tuple, optional): size of target matrix V
+        rank (int, optional): size of hidden dimension
+        **kwargs: arguments passed through to :meth:`BaseComponent <torchnmf.nmf.BaseComponent>`
 
 
     Shape:
@@ -459,8 +471,8 @@ class NMF(BaseComponent):
 
     Examples::
 
-        >>> V = torch.rand(30, 20)
-        >>> m = NMF(V.t().shape, 5)
+        >>> V = torch.rand(20, 30)
+        >>> m = NMF(V.shape, 5)
         >>> m.W.size()
         torch.Size([30, 5])
         >>> m.H.size()
@@ -510,11 +522,15 @@ class NMFD(BaseComponent):
     Note:
         To match with PyTorch convention, an extra batch dimension is required for target matrix V.
 
+    Note:
+        If `Vshape` argument is given, the model will try to infer the size of :meth:`W <torchnmf.nmf.BaseComponent.W>` and
+        :meth:`H <torchnmf.nmf.BaseComponent.H>`, and override arguments passed through to :meth:`BaseComponent <torchnmf.nmf.BaseComponent>`.
+
     Args:
-        Vshape (tuple, optional): Size of target matrix V
-        rank (int, optional): Size of hidden dimension
-        T (int, optional): Size of the convolving window
-        **kwargs: Arguments passed through to :meth:`BaseComponent <torchnmf.nmf.BaseComponent>`.
+        Vshape (tuple, optional): size of target matrix V
+        rank (int, optional): size of hidden dimension
+        T (int, optional): size of the convolving window
+        **kwargs: arguments passed through to :meth:`BaseComponent <torchnmf.nmf.BaseComponent>`
 
 
     Shape:
@@ -585,11 +601,15 @@ class NMF2D(BaseComponent):
     Note:
         To match with PyTorch convention, an extra batch dimension is required for target tensor V.
 
+    Note:
+        If `Vshape` argument is given, the model will try to infer the size of :meth:`W <torchnmf.nmf.BaseComponent.W>` and
+        :meth:`H <torchnmf.nmf.BaseComponent.H>`, and override arguments passed through to :meth:`BaseComponent <torchnmf.nmf.BaseComponent>`.
+
     Args:
-        Vshape (tuple, optional): Size of target tensor V
-        rank (int, optional): Size of hidden dimension
-        kernel_size (int or tuple, optional): Size of the convolving kernel
-        **kwargs: Arguments passed through to :meth:`BaseComponent <torchnmf.nmf.BaseComponent>`.
+        Vshape (tuple, optional): size of target tensor V
+        rank (int, optional): size of hidden dimension
+        kernel_size (int or tuple, optional): size of the convolving kernel
+        **kwargs: arguments passed through to :meth:`BaseComponent <torchnmf.nmf.BaseComponent>`
 
     Shape:
         - V: :math:`(N, C, L_{out}, M_{out})`
@@ -652,11 +672,15 @@ class NMF3D(BaseComponent):
     Note:
         To match with PyTorch convention, an extra batch dimension is required for target tensor V.
 
+    Note:
+        If `Vshape` argument is given, the model will try to infer the size of :meth:`W <torchnmf.nmf.BaseComponent.W>` and
+        :meth:`H <torchnmf.nmf.BaseComponent.H>`, and override arguments passed through to :meth:`BaseComponent <torchnmf.nmf.BaseComponent>`.
+
     Args:
-        Vshape (tuple, optional): Size of target tensor V
-        rank (int, optional): Size of hidden dimension
-        kernel_size (int or tuple, optional): Size of the convolving kernel
-        **kwargs: Arguments passed through to :meth:`BaseComponent <torchnmf.nmf.BaseComponent>`.
+        Vshape (tuple, optional): size of target tensor V
+        rank (int, optional): size of hidden dimension
+        kernel_size (int or tuple, optional): size of the convolving kernel
+        **kwargs: arguments passed through to :meth:`BaseComponent <torchnmf.nmf.BaseComponent>`
 
 
     Shape:
